@@ -5,8 +5,7 @@ from typing import Any, Callable, Dict, List, Optional, TypeVar, Union, cast
 
 from pydantic import root_validator, validator
 
-from hera.shared import global_config
-from hera.shared._base_model import BaseMixin
+from hera.shared import BaseMixin, global_config
 from hera.shared.serialization import serialize
 from hera.workflows._context import SubNodeMixin, _context
 from hera.workflows.artifact import Artifact
@@ -436,8 +435,11 @@ class CallableTemplateMixin(ArgumentsMixin):
             # these are the already set parameters. If a users has already set a parameter argument, then Hera
             # uses the user-provided value rather than the inferred value
             arguments = self.arguments if isinstance(self.arguments, list) else [self.arguments]  # type: ignore
+            arguments = list(filter(lambda x: x is not None, arguments))
             parameters = [arg for arg in arguments if isinstance(arg, ModelParameter) or isinstance(arg, Parameter)]
             parameter_names = {p.name for p in parameters}
+            artifacts = [arg for arg in arguments if isinstance(arg, ModelArtifact) or isinstance(arg, Artifact)]
+            artifact_names = {a.name for a in artifacts}
             if "source" in kwargs and "with_param" in kwargs:
                 # Argo uses the `inputs` field to indicate the expected parameters of a specific template whereas the
                 # `arguments` are used to indicate exactly what _values_ are assigned to the set inputs. Here,
@@ -447,7 +449,7 @@ class CallableTemplateMixin(ArgumentsMixin):
                 new_parameters = _get_params_from_source(kwargs["source"])
                 if new_parameters is not None:
                     for p in new_parameters:
-                        if p.name not in parameter_names:
+                        if p.name not in parameter_names and p.name not in artifact_names:
                             arguments.append(p)
             elif "source" in kwargs and "with_items" in kwargs:
                 # similarly to the above, we can infer the arguments to create based on the content of `with_items`.
@@ -586,3 +588,19 @@ def _get_params_from_items(with_items: List[Any]) -> Optional[List[Parameter]]:
         else:
             return [Parameter(name=n, value=f"{{{{item.{n}}}}}") for n in el.keys()]
     return [Parameter(name=n, value=f"{{{{item.{n}}}}}") for n in with_items[0].keys()]
+
+
+class ExperimentalMixin(BaseMixin):
+    _experimental_warning_message: str = (
+        "Unable to instantiate {} since it is an experimental feature."
+        ' Please turn on experimental features by setting `hera.shared.global_config.experimental_features["{}"] = True`.'
+        " Note that experimental features are unstable and subject to breaking changes."
+    )
+
+    _flag: str
+
+    @root_validator
+    def _check_enabled(cls, values):
+        if not global_config.experimental_features[cls._flag]:
+            raise ValueError(cls._experimental_warning_message.format(cls, cls._flag))
+        return values
